SystemOrganization addCategory: #'Volunti-ElectionManager-Model'!
SystemOrganization addCategory: #'Volunti-ElectionManager-View'!
SystemOrganization addCategory: #'Volunti-ElectionManager-Tests'!

VOAdministrationComponent subclass: #VOElectionManagerAdministrationComponent
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOElectionManagerAdministrationComponent methodsFor: 'private' stamp: 'NicolasVanderAuwera 11/14/2012 13:56'!
buildAuthorityConfigurationComponent
	^ VOElectionManagerAuthorityConfigurationComponent new! !

!VOElectionManagerAdministrationComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/14/2012 13:51'!
components
	^ super components
		at: 'User management' put: editor;
		at: 'Cryptographic parameters' put: VOCryptographicParametersComponent new;
		yourself! !

!VOElectionManagerAdministrationComponent methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 11/14/2012 13:51'!
initialize
	super initialize.
	editor := VOUserEditor new.
	self session announcer
		on: RefreshContent
		do: [ :it | self call: it content ]! !

TestCase subclass: #ElectionTest
	instanceVariableNames: 'anElection emptyElection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-Tests'!

!ElectionTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/1/2012 07:22'!
choices
	^ #('Nicolas' 'Jerome' 'Smith' 'Thomas' 'Amlie' 'Jean' 'Christine')! !

!ElectionTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/1/2012 07:39'!
name
	^ 'test'! !

!ElectionTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 4/7/2012 18:09'!
setUp
	anElection := Election new
		numberOfChoices: self choices size;
		name: self name;
		protocol: VOEMProtocol new;
		yourself.
	self choices do: [ :choice | anElection addChoice: choice ].
	emptyElection := Election new name: self name! !

!ElectionTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 8/20/2012 11:50'!
testAsClientElection
	| clientElection |
	clientElection := anElection asClientElection .
	self assert: clientElection id = anElection id.
	self assert: 'simple' = anElection protocol class key! !

!ElectionTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 3/1/2012 07:23'!
testChoices
	self
		assert: ((OrderedCollection newFrom: self choices) includesAllOf: (anElection choices collect: [ :each | each name ]))! !

!ElectionTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 3/1/2012 07:23'!
testNewBallot
	! !

!ElectionTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 3/1/2012 15:17'!
testValidate
	self should: [ emptyElection validate ] raise: WAValidationNotification.
	self shouldnt: [ anElection validate ] raise: WAValidationNotification! !

ElectionTest subclass: #KOutOfNElectionTest
	instanceVariableNames: 'election2 election3 election4'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-Tests'!

!KOutOfNElectionTest methodsFor: 'class membership' stamp: 'NicolasVanderAuwera 4/7/2012 18:09'!
ballotClassName
	^ KOutOfNElection! !

!KOutOfNElectionTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 8/20/2012 11:56'!
setUp
	anElection := self ballotClassName new numberOfChoices: 3; maxNumberOfAllowedChoices: 2; yourself.
	election2 := self ballotClassName new numberOfChoices: 3; maxNumberOfAllowedChoices: self choices size + 1; yourself.
	election3 := self ballotClassName new numberOfChoices: 3; maxNumberOfAllowedChoices: -1; yourself.
	election4 := self ballotClassName new numberOfChoices: 3; maxNumberOfAllowedChoices: 'aString'; yourself.
	emptyElection := self ballotClassName new.
	self choices
		do: [ :choice | 
			anElection
				name: self name;
				addChoice: choice.
			election2
				name: self name;
				addChoice: choice.
			election3
				name: self name;
				addChoice: choice.
			election4
				name: self name;
				addChoice: choice ]! !

!KOutOfNElectionTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 11/20/2011 14:39'!
testValidate
	self shouldnt: [ anElection validate ] raise: WAValidationNotification.
	self should: [ election2 validate ] raise: WAValidationNotification.
	self should: [ election3 validate ] raise: WAValidationNotification.
	self should: [ election4 validate ] raise: WAValidationNotification! !

KOutOfNElectionTest subclass: #MPtKOutOfNElectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-Tests'!

!MPtKOutOfNElectionTest methodsFor: 'class membership' stamp: 'NicolasVanderAuwera 4/7/2012 18:09'!
ballotClassName
	^ MPtKOutOfNElection! !

!MPtKOutOfNElectionTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 4/7/2012 18:09'!
setUp
	super setUp.
	anElection pointsToDistribute: 2.
	election2 pointsToDistribute: 2.
	election3 pointsToDistribute: -1.
	election4 pointsToDistribute: 'aString'.
	emptyElection := MPtKOutOfNElection new! !

!MPtKOutOfNElectionTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 8/20/2012 11:58'!
testValidate
	self shouldnt: [ anElection validate ] raise: WAValidationNotification.
	self halt.
	self shouldnt: [ election2 validate ] raise: WAValidationNotification.
	self halt.
	self should: [ election3 validate ] raise: WAValidationNotification.
	self should: [ election4 validate ] raise: Error! !

WATask subclass: #VOElectionClosureTask
	instanceVariableNames: 'election'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOElectionClosureTask class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 4/9/2012 11:31'!
election: anElection
	^ self new election: anElection! !

!VOElectionClosureTask methodsFor: 'running' stamp: 'NicolasVanderAuwera 5/2/2012 09:45'!
actionTask: protocol
	protocol close: election with: VOElectionManager current configurations.
	election close! !

!VOElectionClosureTask methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/9/2012 11:31'!
election: anElection
	election := anElection! !

!VOElectionClosureTask methodsFor: 'running' stamp: 'NicolasVanderAuwera 10/29/2012 10:04'!
go
	| protocol |
	protocol := (VOEMProtocol protocolWithKey: election protocol) new.
	self actionTask: protocol.
	self call: (VOLogComponent log: protocol log anchor: 'Continue')! !

VOElectionClosureTask subclass: #VOElectionOpeningTask
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOElectionOpeningTask methodsFor: 'running' stamp: 'NicolasVanderAuwera 5/2/2012 09:48'!
actionTask: protocol
	protocol open: election with: VOElectionManager current configurations.
	election open! !

WATask subclass: #VOElectionCreationTask
	instanceVariableNames: 'protocol selection election'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOElectionCreationTask methodsFor: 'running' stamp: 'NicolasVanderAuwera 11/4/2012 08:15'!
go
	[ 
	election := (self call: selection) new.
	self
		call:
			(election asComponent
				addDecoration: (VOTWBSHeaderDecoration new headerString: 'Configure your election');
				yourself) ] whileNil: [  ].
	[ self session agent sendCreatedElection: election ]
		on: Error
		do: [ :err | 
			self twbsInformError: 'An error occured during the process'.
			self answer: nil ].
	self twbsInformSuccess: 'The election "' , election name , '" is correctly created.'.
	self answer: election! !

!VOElectionCreationTask methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 11/3/2012 17:25'!
initialize
	super initialize.
	selection := WASelection new
		items: Election withAllSubclasses;
		labelBlock: [ :item | item description ];
		addDecoration: (VOTWBSHeaderDecoration new headerString: 'Choose the type of the election');
		yourself! !

WATask subclass: #VOSystemConfigurationTask
	instanceVariableNames: 'manager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOSystemConfigurationTask class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 5/5/2012 11:31'!
manager: aManager
	^ self new manager: aManager! !

!VOSystemConfigurationTask methodsFor: 'running' stamp: 'NicolasVanderAuwera 11/2/2012 12:30'!
go
	[ 
	self call: manager asComponent.
	self call: (VOSystemConfigurationCheck model: manager).
	manager isConfigured ] whileFalse! !

!VOSystemConfigurationTask methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 11:32'!
manager: aManager
	manager := aManager! !

VOAgentComponent subclass: #VOElectionManagerComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOElectionManagerComponent class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/27/2011 13:25'!
url
	^ 'electionmanager'! !

!VOElectionManagerComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/1/2012 12:13'!
administration
	^ VOElectionManagerAdministrationComponent! !

!VOElectionManagerComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/9/2011 07:45'!
agentClass
	^ VOElectionManager! !

!VOElectionManagerComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/17/2012 15:18'!
main
	^ VOElectionManagerMainComponent! !

!VOElectionManagerComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/27/2011 12:59'!
title
	^ 'Election Manager'! !

VOCertifiedAgent subclass: #VOElectionManager
	instanceVariableNames: 'elections manager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-Model'!

VOElectionManager class
	instanceVariableNames: 'protocols'!

!VOElectionManager commentStamp: 'NicolasVanderAuwera 3/1/2012 07:13' prior: 0!
I contains the different Elections.

Instance Variables:
	elections	<(Collection of: VOElection)>

Class Instance Variables:
	protocols	<ProtoObject>!

VOElectionManager class
	instanceVariableNames: 'protocols'!

!VOElectionManager class methodsFor: 'x509' stamp: 'NicolasVanderAuwera 5/5/2012 15:56'!
organisationUnit
	^ 'Election Manager'! !

!VOElectionManager class methodsFor: 'network' stamp: 'NicolasVanderAuwera 12/1/2011 09:23'!
port
	^ 1234! !

!VOElectionManager class methodsFor: 'network' stamp: 'NicolasVanderAuwera 12/31/2011 11:41'!
protocols
	^ protocols
		ifNil: [ 
			protocols := Dictionary new
				at: #mentalbooth put: VOEMMentalBoothProtocol;
				at: #simple put: VOEMProtocol;
				at: #anonymous put: VOEMProtocol;
				yourself ]! !

!VOElectionManager methodsFor: 'private' stamp: 'NicolasVanderAuwera 11/2/2012 11:59'!
buildConfigurations
	| dictionary |
	dictionary := Dictionary new.
	VOManager allSubclasses
		do: [ :authority | dictionary at: authority key put: (VONetworkConfiguration description: authority organisationUnit key: authority key) ].
	dictionary
		at: 'voidgenerationserver'
		put: (VONetworkConfiguration description: 'ID Generation Server' organisationUnit key: 'voidgenerationserver').
	dictionary
		at: 'vokeygenerationserver'
		put: (VONetworkConfiguration description: 'Key Generation Server' organisationUnit key: 'vokeygenerationserver').
	dictionary
		at: 'vocertificationauthority' key
		put: (VONetworkConfiguration description: 'Certification Authority' organisationUnit key: 'vocertificationauthority').
	dictionary
		at: 'vohomomorphicmanager'
		put: (VONetworkConfiguration description: 'Homomorphic Manager' organisationUnit key: 'vohomomorphicmanager').
	^ VOConfigurationManager configurations: dictionary! !

!VOElectionManager methodsFor: 'removing' stamp: 'NicolasVanderAuwera 11/1/2012 14:00'!
clearConfigurations
	manager := self buildConfigurations! !

!VOElectionManager methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/2/2012 12:22'!
configurationCheck
	manager validateConfigurations! !

!VOElectionManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 16:15'!
configurations
	^ manager configurations! !

!VOElectionManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/1/2012 14:15'!
cryptographicParameters
	cryptographicParameters ifNil: [ cryptographicParameters := CryptographicParameters new ].
	^ cryptographicParameters! !

!VOElectionManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 11/3/2012 07:40'!
handle: aMessage
	super handle: aMessage.
	aMessage command = #elections
		ifTrue: [ self sendElections ].
	aMessage command = #election
		ifTrue: [ self sendElection ].
	aMessage command = #candidates
		ifTrue: [ self sendCandidates ].
	aMessage command = #parameters
		ifFalse: [ ^ self ].
	self sendParameters! !

!VOElectionManager methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 11/1/2012 14:00'!
initialize
	super initialize.
	manager := self buildConfigurations! !

!VOElectionManager methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/7/2012 10:10'!
isConfigured
	^ manager isConfigured! !

!VOElectionManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 11:28'!
manager
	^ manager! !

!VOElectionManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 10/29/2012 14:59'!
sendCandidates
	| election |
	election := self electionWithId: stream nextLine greaseInteger.
	stream sendObject: election choices.
	self addInLog: 'Candidates sent'! !

!VOElectionManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 11/4/2012 08:04'!
sendCreatedElection: anElection
	| managersToContact |
	managersToContact := self configurations select: [ :config | anElection protocol agents includes: config key ].
	managersToContact
		do: [ :managerToContact | (VOConnection configuration: managerToContact) send: (VOCreateElectionMessage new content: anElection) ].
	self addInLog: 'The election "' , anElection name , '" was created by all authorities.'! !

!VOElectionManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 11/1/2012 14:09'!
sendElection
	stream sendObject: (Election withId: stream nextLine greaseInteger)! !

!VOElectionManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 11/1/2012 14:09'!
sendElections
	stream sendObject: Election database! !

!VOElectionManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 11/3/2012 10:01'!
sendParameters
	"Send the cryptographic parameters and the configurations to all authorities."

	| parameters |
	parameters := Dictionary new
		at: #configurations put: self configurations;
		at: #parameters put: self cryptographicParameters deepCopy beReadonly;
		yourself.
	self configurations
		do: [ :config | (VOConnection configuration: config) send: (VOCryptographicParametersMessage new content: parameters) ]! !

WAComponent subclass: #VOSystemConfigurationCheck
	instanceVariableNames: 'model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOSystemConfigurationCheck class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 1/17/2012 11:32'!
model: aProtocol
	^ self new model: aProtocol! !

!VOSystemConfigurationCheck methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/17/2012 11:32'!
model
	^ model! !

!VOSystemConfigurationCheck methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/17/2012 11:32'!
model: anObject
	model := anObject! !

!VOSystemConfigurationCheck methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/2/2012 12:39'!
renderActionsOn: html
	html div
		class: 'form-actions';
		with: [ 
					self renderCheckAllOn: html.
					html space.
					self renderCheckOn: html.
					html space.
					self renderBackOn: html.
					self renderTerminateOn: html ]! !

!VOSystemConfigurationCheck methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/28/2012 10:44'!
renderBackOn: html
	html anchor
		class: 'btn';
		callback: [ self answer: false ];
		with: 'Back'! !

!VOSystemConfigurationCheck methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/2/2012 12:39'!
renderCheckAllOn: html
	html anchor
		class: 'btn btn-info';
		callback: [ self model validateAllConfigurations: true ];
		with: 'Validate all configurations'! !

!VOSystemConfigurationCheck methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/2/2012 12:39'!
renderCheckOn: html
	html anchor
		class: 'btn btn-info';
		callback: [ self model validateAllConfigurations: false ];
		with: 'Validate selected configurations'! !

!VOSystemConfigurationCheck methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/3/2012 07:13'!
renderConfig: config on: html
	html
		paragraph: [ 
			html text: 'The ' , config description , ' is '.
			config isConfigured
				ifTrue: [ 
					html text: 'correctly configured.'. html break.
					html text: 'It is listenning on port '.
					html strong: config port ]
				ifFalse: [ 
					html text: 'misconfigured.'.
					html break.
					html strong: ' Message: '.
					html text: config errorMsg ] ]! !

!VOSystemConfigurationCheck methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/3/2012 07:17'!
renderContentOn: html
	self renderInformationOn: html.
	self model configurations
		keysAndValuesDo: [ :agent :config | 
			html div
				class: 'alert';
				class: 'alert-success' if: config isConfigured;
				class: 'alert-danger' if: (config notConfigured and: [ config toCheck ]);
				class: 'alert-warning' if: config toCheck not;
				with: [ self renderConfig: config on: html ] ].
	self renderActionsOn: html! !

!VOSystemConfigurationCheck methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/28/2012 10:39'!
renderInformationOn: html
	self model isConfigured
		ifFalse: [ 
			html div
				class: 'alert alert-info';
				with: [ 
							html
								paragraph:
									'All agents must be correctly configured before the election is created. Click on the back anchor in the bottom of the page to reconfigure agents.' ] ].
	html horizontalRule! !

!VOSystemConfigurationCheck methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/3/2012 08:08'!
renderTerminateOn: html
	self model isConfigured
		ifTrue: [ 
			html space.
			html anchor
				class: 'btn btn-success';
				callback: [ self terminate ];
				with: 'Terminate' ]! !

!VOSystemConfigurationCheck methodsFor: 'actions' stamp: 'NicolasVanderAuwera 11/3/2012 08:08'!
terminate
	self session agent sendParameters.
	self answer! !

WAComponent subclass: #VOSystemInitializationView
	instanceVariableNames: 'protocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOSystemInitializationView class methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 4/9/2012 10:44'!
initializeSystemWith: aProtocol
	| initializer |
	initializer := self new protocol: aProtocol! !

!VOSystemInitializationView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/9/2012 10:47'!
protocol: aProtocol
	protocol := aProtocol! !

!VOSystemInitializationView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 4/9/2012 10:45'!
renderContentOn: html
	self renderLogOn: html! !

!VOSystemInitializationView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 4/9/2012 10:47'!
renderLogOn: html
	html textArea
		readonly: true;
		value: protocol log! !

WAComponent subclass: #VOUserEditor
	instanceVariableNames: 'report rows'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOUserEditor methodsFor: 'actions' stamp: 'NicolasVanderAuwera 11/2/2012 10:55'!
addUser
	self session announcer
		announce:
			(RefreshContent
				content:
					(VOSystemUser new asComponent
						onAnswer: [ :anUser | 
									anUser
										ifNotNil: [ 
											rows add: anUser.
											self refresh ] ];
						yourself))! !

!VOUserEditor methodsFor: 'private' stamp: 'NicolasVanderAuwera 11/2/2012 10:55'!
buildReport
	^ (TWBSMagritteReport rows: rows description: VOSystemUser new magritteDescription)
		addColumn:
				(MACommandColumn new
						addCommandOn: self selector: #editUser: text: 'edit';
						addCommandOn: self selector: #removeUser: text: 'remove';
						yourself);
		yourself! !

!VOUserEditor methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 11/1/2012 12:15'!
children
	^ super children copyWith: self report! !

!VOUserEditor methodsFor: 'actions' stamp: 'NicolasVanderAuwera 11/1/2012 17:23'!
editUser: anUser
	self session announcer announce: (RefreshContent content: anUser asComponent)! !

!VOUserEditor methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 11/2/2012 10:55'!
initialize
	super initialize.
	rows := VOSystemUser database! !

!VOUserEditor methodsFor: 'actions' stamp: 'NicolasVanderAuwera 11/1/2012 12:45'!
refresh
	self report rows: rows! !

!VOUserEditor methodsFor: 'removing' stamp: 'NicolasVanderAuwera 11/1/2012 12:48'!
removeUser: anUser
	rows remove: anUser.
	self refresh! !

!VOUserEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/1/2012 17:26'!
renderButtonOn: html
	html div
		class: 'form-actions form-horizontal';
		with: [ 
					html submitButton
						class: 'btn btn-primary';
						callback: [ self addUser ];
						with: 'Add' ]! !

!VOUserEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/1/2012 12:20'!
renderContentOn: html
	html form class: 'magritte form-horizontal'; with: [ 
		self renderReportOn: html.
		self renderButtonOn: html ]! !

!VOUserEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/1/2012 12:19'!
renderReportOn: html
	html render: self report! !

!VOUserEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/1/2012 11:53'!
report
	^ report ifNil: [ report := self buildReport ]! !

WAChoiceDialog subclass: #VOExtendedChoiceDialog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOExtendedChoiceDialog methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/18/2012 08:17'!
buttons
	^ #(ok)! !

!VOExtendedChoiceDialog methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 1/27/2012 15:09'!
renderContentOn: html
	html heading
		level: 2;
		with: 'Choose the protocol'.
	html paragraph
		id: html nextId;
		class: 'description';
		with: [ html text: self options first description ].
	html select
		list: self options;
		labels: [ :each | each title ];
		selected: self selection;
		callback: [ :value | self selection: value ];
		onChange: ((html jQuery: html lastId) load html: [ :h | h render: self selection description ])! !

VOManagerMainComponent subclass: #VOElectionManagerMainComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOElectionManagerMainComponent methodsFor: 'adding' stamp: 'NicolasVanderAuwera 11/3/2012 13:36'!
addElection
	| election |
	election := self call: VOElectionCreationTask new.
	election ifNotNil: [ :arg | Election add: election ]! !

!VOElectionManagerMainComponent methodsFor: 'private' stamp: 'NicolasVanderAuwera 11/17/2012 15:10'!
buildElectionsListComponent
	^ VOEMElectionsListPage elections: Election database! !

!VOElectionManagerMainComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/3/2012 13:35'!
createElectionTask
	"	self agent configurationCheck.
	self agent isConfigured
		ifTrue: [ self agent addElection: (self call: VOElectionCreationTask new) ]
		ifFalse: [ self inform: 'Some agents are misconfigured. Please verify your configuration befor creating a new election.' ]"

	"Election add: (self call: Election new asComponent)"
	Election add: (self call: VOElectionCreationTask new)! !

!VOElectionManagerMainComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/17/2012 15:06'!
renderConfigurationOn: html
	html
		div: [ 
			html
				paragraph: [ 
					html text: 'You must configure the system before creating elections. Click '.
					self renderConfigureAnchorText: 'here' On: html.
					html text: ' to configure the system.' ] ]! !

!VOElectionManagerMainComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/17/2012 15:06'!
renderConfigureAnchorText: text On: html
	html anchor
		class: 'btn btn-info';
		callback: [ self call: (VOSystemConfigurationTask manager: self agent manager) ];
		with: text! !

!VOElectionManagerMainComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/3/2012 08:49'!
renderCreateOn: html
	html anchor
		class: 'btn btn-success';
		callback: [ self addElection ];
		with: 'New election'! !

!VOElectionManagerMainComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/17/2012 15:07'!
renderMainContentOn: html
	super renderMainContentOn: html.
	self session isAdmin
		ifTrue: [ 
			self agent isConfigured
				ifFalse: [ self renderConfigurationOn: html ] ]! !

!VOElectionManagerMainComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/17/2012 15:06'!
renderRestrictedContentOn: html
	super renderRestrictedContentOn: html.
	html div
		class: 'form-actions form-horizontal';
		with: [ self renderCreateOn: html ]! !

VOElectionsListComponent subclass: #VOEMElectionsListPage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOEMElectionsListPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/27/2012 15:14'!
renderClose: election on: html
	html anchor
		class: 'btn btn-error';
		callback: [ self call: (VOElectionClosureTask election: election) ];
		with: 'Close'! !

!VOEMElectionsListPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/17/2012 13:13'!
renderDetailsOf: election on: html
	html anchor
		callback: [ self call: (VOElectionComponent with: election) ];
		with: 'View Details'! !

!VOEMElectionsListPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 5/2/2012 09:49'!
renderMoreData: election on: html
	html tableData: [ self renderDetailsOf: election on: html ].
	html tableData
		class: 'opened' if: election opened;
		with: [ 
					election opened
						ifTrue: [ self renderClose: election on: html ]
						ifFalse: [ self renderOpen: election on: html ] ].
	election startDate isNil
		ifTrue: [ html tableData: 'None' ]
		ifFalse: [ html tableData: election startDate asDate ].
	election endDate isNil
		ifTrue: [ html tableData: 'None' ]
		ifFalse: [ html tableData: election endDate asDate ]! !

!VOEMElectionsListPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 4/9/2012 14:36'!
renderMoreDataHeadingOn: html
	html tableHeading: 'Details'.
	html tableHeading: 'Action'.
	html tableHeading: 'Start Date'.
	html tableHeading: 'End Date'! !

!VOEMElectionsListPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/27/2012 15:13'!
renderOpen: election on: html
	html anchor
		class: 'btn btn-success';
		callback: [ self call: (VOElectionOpeningTask election: election) ];
		with: 'Open'! !

VOAuthorityConfigurationComponent subclass: #VOElectionManagerAuthorityConfigurationComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-View'!

!VOElectionManagerAuthorityConfigurationComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/14/2012 13:56'!
renderContentOn: html
	super renderContentOn: html.
	html anchor
		class: 'btn btn-info';
		callback: [ self configureSystem ];
		with: 'Configure system'! !

GRObject subclass: #VOEMProtocol
	instanceVariableNames: 'agents configured log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-Model'!

VOEMProtocol subclass: #VOEMAnonymousProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-Model'!

!VOEMAnonymousProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/27/2012 14:23'!
description
	^ 'Provides the following properties: anonymity'! !

!VOEMAnonymousProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/27/2012 11:01'!
key
	^ 'anonymous'! !

!VOEMAnonymousProtocol methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 11/4/2012 08:10'!
initialize
	super initialize.
	agents := agents , (Array with: #voanonymousmanager with: #voregistrationofficeManager with: #vopublicboardmanager)! !

!VOEMAnonymousProtocol methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/4/2012 08:08'!
isAnonymous
	^ true! !

!VOEMAnonymousProtocol methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/5/2012 08:47'!
isSimple
	^ false! !

VOEMAnonymousProtocol subclass: #VOEMMentalBoothProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-Model'!

VOEMMentalBoothProtocol subclass: #VOEMAlternativeMBProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-ElectionManager-Model'!

!VOEMAlternativeMBProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/16/2012 09:00'!
description
	^ 'The interface displays m values that the voter has to bind to candidates, including the voter''s secret key'! !

!VOEMAlternativeMBProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/16/2012 08:58'!
key
	^ 'alternative'! !

!VOEMAlternativeMBProtocol methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/8/2012 11:01'!
ballot
	^ VOMBAlternativeBallot! !

!VOEMAlternativeMBProtocol methodsFor: 'elections' stamp: 'NicolasVanderAuwera 5/8/2012 13:37'!
electionTypes
	^ Array with: AlternativeMBElection! !

!VOEMMentalBoothProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/27/2012 14:24'!
description
	^ 'Provides the following properties: anonymity, coercion-resistance, receipt-freeness'! !

!VOEMMentalBoothProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/27/2012 11:01'!
key
	^ 'mentalbooth'! !

!VOEMMentalBoothProtocol methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/21/2012 10:06'!
ballot
	^ VOMBBallot! !

!VOEMMentalBoothProtocol methodsFor: 'elections' stamp: 'NicolasVanderAuwera 4/7/2012 18:47'!
electionTypes
	^ Array with: MBElection! !

!VOEMMentalBoothProtocol methodsFor: 'initialize-release' stamp: 'NicolasVanderAuwera 11/4/2012 08:09'!
initialize
	super initialize.
	agents := agents
		,
			(Array
				with: #vomentalboothmanager
				with: #vompdvsmanager
				with: #vowitnessmanager
				with: #vohomomorphicmanager)! !

!VOEMMentalBoothProtocol methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/4/2012 08:08'!
isAnonymous
	^ false! !

!VOEMMentalBoothProtocol methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/4/2012 08:08'!
isMentalBooth
	^ true! !

!VOEMProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/16/2012 08:53'!
description
	^ 'No property is supported.'! !

!VOEMProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/16/2012 08:52'!
key
	^ 'simple'! !

!VOEMProtocol class methodsFor: 'searching' stamp: 'NicolasVanderAuwera 4/9/2012 11:44'!
protocolWithKey: aSymbol
	^ self withAllSubclasses detect: [ :protocolClass | protocolClass key = aSymbol ]! !

!VOEMProtocol class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/27/2012 14:22'!
title
	^ self key capitalized! !

!VOEMProtocol methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/17/2012 08:50'!
agents
	^ agents! !

!VOEMProtocol methodsFor: 'network' stamp: 'NicolasVanderAuwera 2/16/2012 08:25'!
ballot
	^ nil! !

!VOEMProtocol methodsFor: 'opening' stamp: 'NicolasVanderAuwera 5/2/2012 13:14'!
close: anElection with: configurations
	self addInLog: 'Closure of the election named ' , anElection name.
	self agents
		do: [ :agent | 
			self addInLog: 'The agent ' , agent title , ' is closing the election.'.
			self sendElectionId: anElection id config: (configurations at: agent key) message: 'close'.
			self addInLog: 'The agent ' , agent title , ' has correctly closed the election.' ]! !

!VOEMProtocol methodsFor: 'elections' stamp: 'NicolasVanderAuwera 4/7/2012 18:48'!
electionTypes
	^ Array with: Election with: KOutOfNElection with: MPtKOutOfNElection! !

!VOEMProtocol methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 11/4/2012 08:05'!
initialize
	super initialize.
	agents := Array with: #vopollingofficemanager! !

!VOEMProtocol methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/4/2012 08:07'!
isAlternative
	^ false! !

!VOEMProtocol methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/4/2012 08:07'!
isAnonymous
	^ false! !

!VOEMProtocol methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/4/2012 08:07'!
isMentalBooth
	^ false! !

!VOEMProtocol methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/5/2012 08:47'!
isSimple
	^ true! !

!VOEMProtocol methodsFor: 'opening' stamp: 'NicolasVanderAuwera 5/2/2012 13:14'!
open: anElection with: configurations
	self addInLog: 'Opening of the election named ' , anElection name.
	self agents
		do: [ :agent | 
			self addInLog: 'The agent ' , agent title , ' is opening the election.'.
			self sendElectionId: anElection id config: (configurations at: agent key) message: 'open'.
			self addInLog: 'The agent ' , agent title , ' has correctly opened the election.' ]! !

!VOEMProtocol methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/2/2012 10:55'!
sendElectionId: anId config: config  message: msg
	| stream |
	stream := SocketStream openConnectionToHostNamed: config host port: config port.
	stream sendCommand: msg.
	stream sendCommand: anId greaseString.
	stream close! !
